@@ -1,11 +1,14 @@
-import json
-import urlparse
-from .process import Process
-from schema_salad.ref_resolver import Loader, ContextType
+from __future__ import absolute_import
+from typing import IO, Any, Dict, Text
+
+from rdflib import Graph
+
 from schema_salad.jsonld_context import makerdf
-from rdflib import Graph, plugin, URIRef
-from rdflib.serializer import Serializer
-from typing import Any, Dict, IO, Text, Union
+from schema_salad.ref_resolver import ContextType
+from six.moves import urllib
+
+from .process import Process
+
 
 def gather(tool, ctx):  # type: (Process, ContextType) -> Graph
     g = Graph()
@@ -16,14 +19,16 @@ def gather(tool, ctx):  # type: (Process, ContextType) -> Graph
     tool.visit(visitor)
     return g
 
+
 def printrdf(wf, ctx, sr, stdout):
     # type: (Process, ContextType, Text, IO[Any]) -> None
     stdout.write(gather(wf, ctx).serialize(format=sr))
 
+
 def lastpart(uri):  # type: (Any) -> Text
     uri = Text(uri)
     if "/" in uri:
-        return uri[uri.rindex("/")+1:]
+        return uri[uri.rindex("/") + 1:]
     else:
         return uri
 
@@ -84,6 +89,7 @@ def dot_with_parameters(g, stdout):  # type: (Graph, IO[Any]) -> None
     for (inp,) in qres:
         stdout.write(u'"%s" [shape=octagon]\n' % (lastpart(inp)))
 
+
 def dot_without_parameters(g, stdout):  # type: (Graph, IO[Any]) -> None
     dotname = {}  # type: Dict[Text,Text]
     clusternode = {}
@@ -129,7 +135,7 @@ def dot_without_parameters(g, stdout):  # type: (Graph, IO[Any]) -> None
                 currentwf = None
 
         if Text(runtype) != "https://w3id.org/cwl/cwl#Workflow":
-            stdout.write(u'"%s" [label="%s"]\n' % (dotname[step], urlparse.urldefrag(Text(step))[1]))
+            stdout.write(u'"%s" [label="%s"]\n' % (dotname[step], urllib.parse.urldefrag(Text(step))[1]))
 
     if currentwf is not None:
         stdout.write("}\n")
@@ -163,7 +169,7 @@ def printdot(wf, ctx, stdout, include_parameters=False):
 
     stdout.write("digraph {")
 
-    #g.namespace_manager.qname(predicate)
+    # g.namespace_manager.qname(predicate)
 
     if include_parameters:
         dot_with_parameters(g, stdout)
